bl_info = {
    "name": "SimpleBake",
    "author": "Lewis <www.toohey.co.uk/SimpleBake/support/>",
    "version": (1, 2, 8),
    "blender": (4, 0, 0),
    "location": "Properties Panel -> Render Settings Tab",
    "description": "Simple baking of PBR and other textures",
    "warning": "",
    "wiki_url": "",
    "tracker_url": "",
    "category": "Object",
}

__version__ = bl_info["version"]
from .download import download_zip
from pathlib import Path
down_path = Path(__file__).parent
download_zip("addons_file/SimpleBake/resources.zip",down_path)

import bpy
from bpy.utils import register_class, unregister_class

from . import object_preperation
from . import image_management
from . import material_management
from . import external_save
from . import property_group
from . import uv_management
from . import copy_and_apply
from . import starting_checks
from . import utils
from . import background_and_progress
from . import channel_packing
from . import channel_packing_old
try:
    from . import auto_update
except:
    auto_update = None
from . import presets
from . import presets_local
from . import sketchfab_upload
from . import messages
    
from . import ui
from . import bake_operators

if auto_update != None:
    from .auto_update import VersionControl
else:
    VersionControl = None


classes = ([
        ])

def register():
    global classes
    for cls in classes:
        register_class(cls)
        
    object_preperation.register()
    image_management.register()
    material_management.register()
    external_save.register()
    property_group.register()
    ui.register()
    uv_management.register()
    copy_and_apply.register()
    bake_operators.register()
    starting_checks.register()
    utils.register()
    background_and_progress.register()
    channel_packing.register()
    channel_packing_old.register()
    if auto_update!=None:
        auto_update.register()
    presets.register()
    presets_local.register()
    sketchfab_upload.register()
    messages.register()
    
    prefs = bpy.context.preferences.addons[__package__].preferences
    
    version = __version__
    if VersionControl != None:
        VersionControl.installed_version = version
        no_online_check = False
        if prefs.no_update_check:
            no_online_check = True
        VersionControl.check_at_current_version(no_online_check)
    

def unregister():
    global classes
    for cls in classes:
        unregister_class(cls)
        
    object_preperation.unregister()
    image_management.unregister()
    material_management.unregister()
    external_save.unregister()
    property_group.unregister()
    ui.unregister()
    uv_management.unregister()
    copy_and_apply.unregister()
    bake_operators.unregister()
    starting_checks.unregister()
    utils.unregister()
    background_and_progress.unregister()
    channel_packing.unregister()
    channel_packing_old.unregister()
    if auto_update != None:
        auto_update.unregister()
    presets.unregister()
    presets_local.unregister()
    sketchfab_upload.unregister()
    messages.unregister()
    
